!> \file bmelt-eurasie-depth-lake_mod.f90
!! Module qui calcule la fusion basale (grounded ou ice shelves)
!<

!> \namespace  bmelt_eurasie_regions
!! Calcule la fusion basale (grounded ou ice shelves)
!! @note Pour les ice shelves Antarctique, tient compte des diffrentes rgions
!! A choisir dans le module_choix
!! \author Catherine
!! \date Juiilet 2005
!! @note Used module
!! @note   - use module3D_phy
!<


module  bmelt_eurasie_regions       ! cat juillet 2005

! calcule la fusion basale (grounded ou ice shelves)
! pour les ice shelves Antarctique, tient compte des diffrentes rgions
! A choisir dans le module_choix

       USE module3D_phy

implicit none


REAL,dimension(nx,ny) ::  bmgrz       !< fusion basale a la grounding zone
real,dimension(nx,ny) ::  bmshelf     !< fusion basale sous shelf

!REAL,dimension(nx,ny) :: dist_talu    ! distance du point au talu continental
!REAL,dimension(nx,ny) :: typeshelf    ! Type de shelf Ronne->1 Ross ->2 ....
                                      ! utilises pour moduler la fusion sous le shelf
!integer, dimension(10) :: region     ! pour crire dans le fichier param
!character(len=30),dimension(10) :: regname ! nom des rgions 
real :: bsupshelf

CONTAINS
!-------------------------------------------------------------------------------

!>
!!SUBROUTINE: init_bmelt
!! Cette routine fait l'initialisation pour la fusion basale.
!! Elle est appele par inputfile-vec-0.5.f90
!<
subroutine init_bmelt

! Cette routine fait l'initialisation pour la fusion basale.
! Elle est appele par inputfile-vec-0.5.f90

!    ecriture dans le fichier parametres
     write(42,*)'fusion basale sous les ice shelves : bmelt-constant_mod'
     write(42,*)'-------------------------------------------------------'

! lecture du fichier contenant les distances au talu continental (m)
!      open(88,file=TRIM(DIRNAMEINP)//'distance_talu-40km.xy')
!      do j=1,ny
!         do i=1,nx
!            read(88,'(i3,1x,i3,1x,f10.2)') k,k,dist_talu(i,j)
!         enddo
!      enddo
!      close(88)

! lecture du fichier contenant les types de shelves
!  Ronne-Flichner ->1, Ross -> 2 , Amery -> 3, 
!  PIG-> 4, les petits shelves au dessus de PIG -> 5

!      open(88,file=TRIM(DIRNAMEINP)//'numer-ice-shelves-av05')
!      typeshelf(:,:)=100
!      do k=1,nx*ny
!        read(88,*,end=36) i,j,typeshelf(i,j)
!      end do
!36      close(88)


! region(:)=0
! regname(1)='Ronne-Fichner'
! regname(2)='Ross'
! regname(3)='Amery'
! regname(4)='Pig'
! regname(5)='Petits ice shelves peninsule'
! regname(6)='autres ice shelves'
! regname(7)='Au dela du talus continental'
 
bms_init:    do j=1,ny
         do i=1,nx

            if ((sealevel_2d(i,j)-BSOC(i,j)).lt.450) then
                    bmshelf(i,j)=2.00 !3.00! 2.55! 0.45!   0.65  
                    bmgrz(i,j)=2.00 !3.00! 2.55! 0.45 !  0.65
!                   if (i.lt.141.and. j.lt.121) then
!                  bmshelf(i,j)=0.55!   0.65  
!                  bmgrz(i,j)=-1.55 !  0.65
!                   endif        
            else         
                    bmshelf(i,j)=8.15 ! 1.45!   0.65  
                    bmgrz(i,j)=8.15 ! 1.45 !  0.65
            endif  

         enddo
      enddo bms_init


      bsupshelf=.2
      write(32,*)


      return
      end subroutine init_bmelt

!________________________________________________________________________________

!>
!!SUBROUTINE: bmeltshelf
!!Cette routine calcule la fusion basale proprement dite
!<
subroutine bmeltshelf


! cette routine calcule la fusion basale proprement dite

integer :: ngr           ! nombre de voisins flottants

print*,coefbmshelf

    do j=2,ny-1
      do i=2,nx-1
 

                   if (sealevel_2d(i,j).gt.0) then
                         bmshelf(i,j)=0.05!5.0    
                         bmgrz(i,j)=0.05!5.0
                   endif 
                   
shelf:    if (flot(i,j)) then    ! partie flottante
        
                     
             if (sealevel_2d(i,j).gt.sealevel) then
                     bmelt(i,j)=bmshelf(i,j)
                     if (fbm(i,j))  bmelt(i,j)=bmgrz(i,j)
             else       
                     bmelt(i,j)=coefbmshelf*bmshelf(i,j)
                     if (fbm(i,j))  bmelt(i,j)=coefbmshelf*bmgrz(i,j)
             endif 
!                           bmelt(i,j)=bmshelf(i,j)
!                  if (time.gt.5000.) then
!                     bmelt(i,j)=bmelt(i,j)+bsupshelf
!                  endif    

! ATTENTION LE BLOC SUIVANT SERT A FAIRE DES ICE SHELVES STATIONNAIRES
! igrdline est dfini dans itnitial1

  if (igrdline.eq.1) then
      corrbmelt(i,j)=corrbmelt(i,j)+hdot(i,j)*0.8
      bmelt(i,j)=bmelt(i,j)+corrbmelt(i,j)
  endif


        else                   ! point pos, on compte le nombre de voisins flottants
           ngr=0
           if (flot(i+1,j)) ngr=ngr+1
           if (flot(i-1,j)) ngr=ngr+1
           if (flot(i,j+1)) ngr=ngr+1
           if (flot(i,j-1)) ngr=ngr+1

!   la fusion des points limites est une combinaison entre valeur pose et valeur flottante
!   en fonction du nombre de points flottants

           if (sealevel_2d(i,j).gt.sealevel) then
                 bmelt(i,j)= ngr/4.*bmgrz(i,j)+(1.-ngr/4.)*bmelt(i,j)         
           else
                 bmelt(i,j)= ngr/4.*bmgrz(i,j)*coefbmshelf+(1.-ngr/4.)*bmelt(i,j)
           endif       

           

        endif shelf

      end do 
    end do


    return
    end subroutine  bmeltshelf


end module  bmelt_eurasie_regions       ! cat juillet 2005
