!> \file climat-forcage-stat-mois_mod-0.1.f90
!! Module pour forcage avec champs mensuels stationnaire
!<

!> \namespace climat_forcage_stat_mois_mod
!!Module pour forcage avec champs mensuels stationnaire
!! \author ...
!! \date ...
!! @note Used modules:
!! @note   - use module3D_phy
!! @note   - use printtable
!<
module climat_forcage_stat_mois_mod

       USE module3D_phy
       USE printtable
       
  IMPLICIT NONE

! 1=decalaration variables
!-------------------------

integer :: nft 
integer :: mo,ti,tj,la
integer :: lake(nx,ny)					!< NFT est le nombre de lignes a lire dans le fichier contenant le forcage climatique
integer,parameter :: mois=12
integer,parameter :: NTR=1 				!< nb of snapshot files : 1 pour stationnaire - ntr is now explicitely specified in the climat module
 			
real,dimension(:),allocatable :: TDATE          	!< time for climate forcing
real,dimension(:),allocatable :: alphaT
real,dimension(:),allocatable :: alphaP
real,dimension(:),allocatable :: SPERT
real ttr(ntr)						!< la date des tranches (snapshots)  (len=ntr) 
!reak alphaTTR(ntr) ! Le alphaT de l'index glaciologiq. aux snapshots

real Tm(nx,ny,mois,ntr),Pm(nx,ny,mois,ntr)
real Tm_time(nx,ny,mois),Pm_time(nx,ny,mois)
real Tm_surf(nx,ny,mois+1) 				!< surface temperature (after topo. correction)
real lapserate(nx,ny,mois)

character (len=100) :: filin, filin2
character (len=100) :: filtr_t(ntr), filtr_p(ntr),filtr_t1, filtr_p1 !< snapshot file name (len=ntr)



! 2=lecure des inputs
!--------------------
contains

!> SUBROUTINE: input_clim
!!  Routine qui permet d'initialiser les variables climatiques
!>
  SUBROUTINE input_clim 					!routine qui permet d'initialiser les variables climatiques

    USE module3D_phy

    IMPLICIT NONE

    character(len=8) :: control				!label to check clim. forc. file (filin) is usable
    integer :: l 						!In snapshot files:the first column is the mask, read but not used


    !Pm(:,:,:,:)=0.
    !Pm_time(:,:,:)=0.
    !Tm(:,:,:,:)=0.
    !Tm_time(:,:,:)=0.
    lapserate(:,:,:)=0.008


    !====================================================HEMINOR============
    if (geoplace.eq.'hemin40') then

       !     atmospheric temperature gradient
       TEMPGRAD=0.008
       TEMPGRJUL=0.0065

       ! 1_fichiers snapshots pour forcage
       !----------------------------------
       !           filtr(1)=TRIM(DIRNAMEINP)//'FORCAGES/forcage_hemin40-21k.g40'
       !           filtr(2)=TRIM(DIRNAMEINP)//'FORCAGES/file_regions_2act.g40'

       ! 2_fichiers donnant l'evolution temporelle
       ! -----------------------------------------
       !        filin=TRIM(DIRNAMEINP)//'signal-cycle-hemin.dat'


       !====================================================EURASIE============
    elseif (geoplace.eq.'euras40') then

       !     atmospheric temperature gradient
       !       TEMPGRAD=0.008
       !       TEMPGRJUL=0.0065
       TEMPGRAD=0.006 ! Pour Serie 2 (grad T reduit)
       TEMPGRJUL=0.005


       filtr_t(1)=TRIM(DIRNAMEINP)//'/ABSOLU/forc_140ka_12temper.dat'     ! 1_fichiers snapshots pour forcage
       filtr_p(1)=TRIM(DIRNAMEINP)//'/ABSOLU/forc_140ka_12precip.dat'

       filin=TRIM(DIRNAMEINP)//'signal-seal_i_inso_90ka.data'		! 2_fichiers donnant l'evolution temporelle
       filin=TRIM(DIRNAMEINP)//'signal-sealev_inso_26jui06.dat'

       filin2=TRIM(DIRNAMEINP)//'lak_reg40km_eu40_ij'


       !====================================================ANT20 ET 40========
    elseif ((geoplace.eq.'anteis1').or.(geoplace.eq.'ant20km')) then


       !     atmospheric temperature gradient
       TEMPGRAD=0.0085
       TEMPGRJUL=0.0085

       ! 1_fichiers snapshots pour forcage
       !----------------------------------
       !            filtr(1)='../INPUT-DATA/FORCAGES/forcage-ant-20k.g40'
       print*,'lecture de ',filin,filtr_p,filtr_t
       ! 2_fichiers donnant l'evolution temporelle
       ! ---------------------------- ------------
       filin=TRIM(DIRNAMEINP)//'forcmodif4cycles.dat' !forcage vostok en rapport : a creer

    endif !Fin du test sur geolpace

    ! 3_lecure des fichiers snapsots pour tout geoplace
    ! -------------------------------------------------
    write(6,*) 'fichiers snapshots'
    do k=1,ntr
       !temperature
       write(6,*) filtr_t(k)
       open(20,file=filtr_t(k))
       do j=1,ny
          do i=1,nx
             read(20,*) ti, tj, (Tm(i,j,mo,k),mo=1,12)
          end do
       end do
       !precipitation
       close(20)
       close(21)
       write(6,*) filtr_p(k)
       open(20,file=filtr_p(k))
       do j=1,ny
          do i=1,nx
             read(20,*) ti, tj, (Pm(i,j,mo,k),mo=1,12)
          end do
       end do
       close(20)
    end do
    !       ttr(1)=-500000. !ttr est la date des tranches 3D-1.h
    !       ttr(2)=0
    ! 4_lecure des fichiers d'evolution pour tout geoplace
    ! ----------------------------------------------------
    open(20,file=filin,status='old')
    !        print*,nft
    read(20,*) control,nft
    print*,'control',control,nft
    ! Determination of file size (line nb), allocation of perturbation array
    if (control.ne.'nb_lines') then
       write(6,*) filin,'indiquer le nb de ligne en debut de fichier:'
       write(6,*) 'le nb de lignes et le label de control nb_lines'
       stop
    endif

    if (.not.allocated(tdate)) THEN
       allocate(TDATE(nft),stat=err)
       if (err/=0) then
          print *,"Erreur  l'allocation du tableau TDATE",err
          stop 4
       end if
    end if

    if (.not.allocated(alphat)) THEN
       allocate(alphaT(nft),stat=err)
       if (err/=0) then
          print *,"Erreur  l'allocation du tableau TPERT",err
          stop 4
       end if
    end if

    if (.not.allocated(alphap)) THEN
       allocate(alphap(nft),stat=err)
       if (err/=0) then
          print *,"Erreur  l'allocation du tableau TPERT",err
          stop 4
       end if
    end if

    if (.not.allocated(spert)) THEN
       allocate(spert(nft),stat=err)
       if (err/=0) then
          print *,"Erreur  l'allocation du tableau SPERT",err
          stop 4
       end if
    end if

    do I=1,NFT
       !          read(20,*) TDATE(I),alphaT(I),alphaP(i),SPERT(I) ! forc grip
       !          print*,i,TDATE(I),alphaT(I),alphaP(i),SPERT(I)
       read(20,*) TDATE(I),alphaT(I),SPERT(I)           ! forc inso
    end do
    !RAJOUT VINCE
    !        do k=1,ntr
    !         do I=1,NFT
    !           if (TDATE(I)==ttr(k)) then
    !             alphaTTR(k)=alphaT(I)
    !           endif
    !         end do
    !        end do

    alphaP(:)=1.0
    !----------------------------masque de glace pour l'Eurasie
    do j=3,ny-2
       do i=3,nx-2
          mk0(i,j)=1
          if ((i.lt.25).and.(j.gt.35)) mk0(i,j)=0
       end do
    end do
    !----------------------------pour les lacs
    !print*, 'module forcage climatique'
    !print*, 'ajout de lakes'

    !   open(50,file=filin2)
    !       do j=1, ny
    !          write(*,*) 'entree boucle j'
    !          do i=1, nx
    !	    write(*,*) 'entree boucle i'
    !	    write(*,*) i
    !            read(50,*) k, k, lake(i,j)
    !          enddo
    !       enddo   
    !   close(50)

    ! A CREER ou organiser a partir des codes dans ! 2_fichiers donnant l'evolution temporelle

  end subroutine input_clim
!--------------------------------------------------------------------------------
!> SUBROUTINE: init_forclim
!!  Routine qui permet d'initialiser les variables climatiques au cours du temps
!>
SUBROUTINE init_forclim

!namelist/clim_forc/
!namelist/clim_pert/rappact,retroac,rapbmshelf,mincoefbmelt,maxcoefbmelt

!rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
!read(num_param,clim_forc)

! formats pour les ecritures dans 42
428 format(A)

write(num_rep_42,428)'!___________________________________________________________' 
write(num_rep_42,428) '&clim_forc_mensuel ! pour le forcage climatique en stationnaire'
write(num_rep_42,*)
end subroutine init_forclim
!---------------------------------------------------------------------
!forcage climatique au cours du temps
!> SUBROUTINE: forclim  
!!  Routine permet d'effectuer le calcul climatique au cours du temps
!>
subroutine forclim

  implicit none
  real COEFT,COEFP,coeftime!coeftime is not used
  INTEGER L !dumm index for loops on snapsots files  l=ITR,NTR-1
  INTEGER ITR !nb of the current snapshot file (change with time)
!       time en dehors des limites du fichier forcage
INTEGER ii,jj
              ITR=1
!
!run mensuels stationnaires
! avec 1 snapshots
do mo=1,mois
    Tm_time(:,:,mo)=Tm(:,:,mo,1)
    Pm_time(:,:,mo)=Pm(:,:,mo,1)
end do

!if(time.le.tdate(1)) then   ! time avant le debut du fichier forcage
!   sealevel=spert(1)
!   coeft=alphat(1)
!   coefp=alphap(1)
!   ift=1
!
!else if (time.ge.tdate(nft)) then     ! time apres la fin du fichier forcage
!   sealevel=spert(nft)
!   coeft=alphat(nft)
!   coefp=alphap(nft)
!   ift=nft
!
!else
!
!
!!A modifier
!    ift = 1      ! modifie par SC le 24/11/99
!   do i=ift,nft-1
!!       print*,'ds boucle1 sur snapshots',ift,TDATE(I),TDATE(I+1),time 
!          if((time.ge.tdate(i)).and.(time.lt.tdate(i+1))) then
!
!!        interpolation entre I et I+1 : cas general
!!           TAFOR=TPERT(I)+(TPERT(I+1)-TPERT(I))*
!!    &            (time-TDATE(I))/(TDATE(I+1)-TDATE(I))
!          SEALEVEL=SPERT(I)+(SPERT(I+1)-SPERT(I))*    &
!                 (time-tdate(I))/(tdate(I+1)-tdate(I))
!
!          coeft=alphaT(I)+(alphaT(I+1)-alphaT(I))*      &
!                 (time-tdate(I))/(tdate(I+1)-tdate(I))
!
!!          COEFP=alphaP(I)+(alphaP(I+1)-alphaP(I))*      &
!!                 (time-TDATE(I))/(TDATE(I+1)-TDATE(I))
!
!                 do k=1,ntr-1  
!                   if((time.ge.ttr(k)).and.(time.lt.ttr(k+1))) goto 105
!
!                 enddo
!105  continue
!
!if (k==ntr) k=ntr-1
!!Rajout Vince 26 juin 2006
!! coefbmshelf est donn avec l'index d'insolation  65N
!          coefbmshelf=coefT
!!On retablit l'index entre 
!
!                  if((time.ge.ttr(1)).and.(time.lt.ttr(ntr))) then
!                      coeft  =   ( coeft - alphaTTR(k) ) / &
!                           ( alphaTTR(k+1)-alphaTTR(k) )
!                  else
!                      print*,'on est pas dans les priodes forc'
!                      print*,'les bornes sont ttr(1) et ttr(ntr)',ttr(1),ttr(ntr)
!                      print*,'time=',time
!                      stop
!                  endif
!
!
!
!! Pour les experiences eurasiennes coefp==coeft
!          COEFP=COEFT
!!        SEALEVEL=-50.
!            IFT=I
!            goto 100
!          endif
!
!        end do
!      endif
!100   continue
!
!!          print*,'coeffT P',COEFT,COEFP
!!=forcage du climat
!!       time en dehors des limites du fichier forcage
!        if(time.le.Ttr(1)) then   ! mis a -500 000 ans
!          do j=1,ny
!            do i=1,nx
!              delTatime(i,j)=delTa(i,j,1)
!              delTjtime(i,j)=delTj(i,j,1)
!              rapactime(i,j)=rapact(i,j,1)
!            end do
!           end do
!              ITR=1
!        else if (time.ge.ttr(Ntr)) then        ! mis a 0
!          do j=1,ny
!            do i=1,nx
!             delTatime(i,j)=delTa(i,j,ntr)
!             delTjtime(i,j)=delTj(i,j,ntr)
!             rapactime(i,j)=rapact(i,j,ntr)
!            end do
!           end do
!           ITR=NTR
!        else               !  interpolation entre l et l+1 : cas general
!
!        itr=max(itr,1)
!       WRITE(6,*)'itr = !',itr
!
!
!!parametres du fit
!        do l=ITR,NTR-1
!        print*,'ds boucle2 sur snapshots',l,ttr(l) 
!           if((time.ge.ttr(l)).and.(time.lt.ttr(l+1))) then  ! test tranche
!
!           coeftime= (time-TTR(l))/(TTR(l+1)-TTR(l))
!           do j=1,ny
!            do i=1,nx
!            delTatime(i,j)=delTa(i,j,l)+             &
!               (delTa(i,j,l+1)-delTa(i,j,l))*coefT
!
!            delTjtime(i,j)=delTj(i,j,l)+             &
!               (delTj(i,j,l+1)-delTj(i,j,l))*coefT
!
!            rapactime(i,j)=rapact(i,j,l)+            &
!               (rapact(i,j,l+1)-rapact(i,j,l))*coefP
!             end do
!           end do
!              print*,'l= ',l,delTj(40,30,l),delTj(40,30,l+1),delTjtime(40,30)
!           ITR=l
!            goto 200
!
!            endif        ! fin du test sur la tranche
!         end do
!       endif   ! fin du test avant,apres,milieu
!
!200   continue
print*,'l= ',l,'itr= ',itr

!--------NIVEAU DES MERS ET DES LAKES

if ((time.lt.-95000.).or.(time.gt.-85500.)) then ! time SANS lacs
                levelflot=sealevel
else
   do j=1,ny  
     do i=1,nx    
!!if (ice(i,j)==1) delTjtime(i,j)=delTjtime(i,j)-3. ! correction effet albedo
!             if ( lake(i,j)==2 ) then     !KOMI
!               levelflot(i,j)=100.
!             elseif ( lake(i,j)==3 ) then !SIBERIAN
!               levelflot(i,j)=60.
!               if ((.not.flot(i,j)).and.(flot(i+1,j)) ) print*,i,j,h(i,j),bmelt(i,j)
!            else
!               levelflot(i,j)=sealevel
!            endif
             levelflot(i,j)=-60.
     enddo
   enddo 
endif
     levelflot(:,:)=max(levelflot(:,:),bsoc(:,:))
   nom_table='flot'
   call printtable_l(flot,nom_table)
   nom_table='lvflo'
   call printtable_r(levelflot,nom_table)

!   coefbmshelf(:,:)=(1.+delTatime(:,:)/7.)  
!coefbmshelf=1. ! modif pour test de sensibilite (tof 20 avril 01)
!coefbmshelf=coefT
    coefbmshelf=max(coefbmshelf,0.1)
    coefbmshelf=min(coefbmshelf,2.)
!tafor=??

print*,'time, coefbmshelf ',time, coefbmshelf
print*,'SEA,COEFT,COEFP', SEALEVEL,COEFT,COEFP
!rint*,'= t,sea,tafor',time,sealevel,tafor
print*,'==========================='

!     calcul de Tjuly et et Tann
! the calcul is now done in subroutine forclim
      do J=1,NY
        do I=1,NX
           ZS(I,J)=max(levelflot(i,j),S(I,J))
            do mo=1,mois
           Tm_surf(i,j,mo)= - lapserate(i,j,mo) * (Zs(i,j)-S0(i,j)) & ! correction d'altitude
                      + Tm_time(i,j,mo)

            end do
        end do
      end do

!
  Tm_surf(:,:,mois+1)=Tm_surf(:,:,1)

if ((geoplace(1:5).eq.'hemin').or.(geoplace(1:5).eq.'euras')) then

call accum_month() ! ds le main
call ablation_month()

elseif ((geoplace.eq.'anteis1').or.(geoplace.eq.'ant20km')) then
! call n'est plus dans le main
!call massb_anteis_forcage()
!call ablation()
else
 print*,"partie en travaux climat forcage auter que sur heminord"
 print*,"geoplace ",geoplace
endif
! call bla2
! call bla3

END subroutine forclim

end module climat_forcage_stat_mois_mod
